#!/usr/bin/python

from numpy import *
from configobj import ConfigObj
import sys


#Config-File einlesen

if len(sys.argv)>1: 
	filename=sys.argv[1]
else:
	filename='swarm.conf'

config=ConfigObj(filename)

funct=config['func']				#Zu optimierende Funktion

plot=int(config['plot'])			#Plot ein/aus
resp=(int(config['resx']),int(config['resy']))	#Plot aufloeseung
smooth=float(config['smooth'])			#0 keine Rotation >0 Anzahl Schritte fuer Rotation pro Iteration
rsp=int(config['rsp'])				#Rotationsgeschwindigkeit in Grad/Iteration
verb=int(config['verb'])			#Verbose
res=float(config['res'])			#Aufloesung der Plotoberflaeche
sf=float(config['sf'])				#Groesse der Schwaermer-Kugeln (scalefactor)

n_dim=int(config['n_dim'])			#Anzahl Dimensionen
maxit=int(config['maxit'])			#Anzahl Iterationen
smax=float(config['smax'])			#Feldgroesse in jede Dimension +/- von Ursprung
n_swarm=int(config['n_swarm'])			#Anzahl Schwaermner
v_max=float(config['v_max'])			#Maximal erreichbare Geschwindigkeit der Schwaermer
opt=[float(config['opt1']),float(config['opt2']),float(config['opt3'])] #Optimierungsparameter
sizef=float(config['sizef'])			#Feldgroessen individuell
neigh_s=int(config['neigh_s'])			#Neighborhood groesse
wk=float(config['wk'])				#Traegheit

azimuth=float(config['azimuth'])		#Ansichtsparameter
elevation=float(config['elevation'])
distance=float(config['distance'])

zscale=float(config['zscale'])			#Skalierfaktor fuer z


if sizef==0: sizef=array(n_dim*[[-smax,smax]])

if n_dim>3: plot=0

if plot==1: from enthought.mayavi import mlab

#zu untersuchende Funktion
def testfunc(x):
	#f= x[0]**2 + x[1]**2 #+ sin(x[2]**2) + cos(x[3])
	#f=sin(x[0])+sin(x[1])
	#f=x[0]**2+x[0]**3+x[0]**4+x[1]**2+x[1]**3+x[1]**4
	#f=418.9829*2-x[0]*sin((abs(x[0]))**(1/2.0))-x[1]*sin((abs(x[1])**(1/2.0)))
	if funct=='0':
		f=418.9892*n_dim
		for i in range(n_dim):
			f-=x[i]*sin(sqrt(abs(x[i])))

	elif funct=='1':
		f=10*n_dim
		for i in range(n_dim):
			f+= x[i]**2 - 10*cos(2*pi*x[i])
	else:
		f=eval(funct)
	return f


#aktualisiert Ansicht und fuehrt Drehung aus falls angegeben
def forceview(rot=0):
	if smooth>0 and rot>0:
		currot=mlab.view()[0]
		for step in range(smooth):
			nrot=((rot)/smooth)*step+currot
			mlab.view(nrot,elevation,distance,zeros(3))
	else:
		mlab.view(azimuth,elevation,distance,zeros(3))

rand=random.random

#Positionen und Geschwindigkeiten initialisieren
swarmvel=zeros((n_swarm,n_dim))
swarmpos=zeros((n_swarm,n_dim))
cur_swarmval=array([nan]*n_swarm)

for sw in range(n_swarm):
	for dim in range(n_dim):
		swarmpos[sw][dim]=random.uniform(sizef[dim][0],sizef[dim][1])


#Surface und mlab initialisieren
if plot==1:
	fig=mlab.figure(size=resp)
	xsurf,ysurf=mgrid[sizef[0][0]:sizef[0][1]:(sizef[0][1]-sizef[0][0])/res,sizef[1][0]:sizef[1][1]:(sizef[1][1]-sizef[1][0])/res]
	s=mlab.surf(xsurf,ysurf,testfunc((xsurf,ysurf))/float(zscale),opacity=0.5,figure=fig)

	forceview()
	
	



#Darstellung initialisieren
if plot==1:
	if n_dim==3:
		points=mlab.points3d(swarmpos[:,0],swarmpos[:,1],swarmpos[:,2],scale_factor=sf,figure=fig)
	elif n_dim==2:
		cur_swarmval=testfunc((swarmpos[:,0],swarmpos[:,1])) #Z-Positionen berechnen
		points=mlab.points3d(swarmpos[:,0],swarmpos[:,1],cur_swarmval/float(zscale),scale_factor=sf,figure=fig)
	forceview()


#Bestwerte initialisieren
global_best_val=inf
global_best_pos=array(n_dim*[None])
personal_best_pos=array([n_dim*[None]]*n_swarm)
personal_best_val=array([inf]*n_swarm)
val_sw=[]
neigh_best_pos=array([n_dim*[float64(0)]]*n_swarm)

#Hauptschleife
for it in range(maxit):
	#Statusanzeige
	if verb>0:
		sys.stdout.write("%2i%% Best: %s\r" %((float(it)/maxit*100), global_best_val))
		sys.stdout.flush()
	
	#Vorberechnen fuer Plot
	cur_swarmval=testfunc(transpose(swarmpos))		
	
	#Plot aktualisieren	
	if plot==1:
		points.mlab_source.x=swarmpos[:,0]
		points.mlab_source.y=swarmpos[:,1]
		if n_dim==2:
			points.mlab_source.z=cur_swarmval/float(zscale)
		if n_dim==3:
			points.mlab_source.z=swarmpos[:,2]/float(zscale)
		forceview(rot=rsp)
		#sleep(1)	
	
	#Index des aktuell kleinsten Wertes
	i_min=cur_swarmval.argmin()
	
	#Kleiner als global_best?
	if cur_swarmval[i_min] < global_best_val:
		global_best_val=cur_swarmval[i_min]
		global_best_pos=swarmpos[i_min].copy()
	
	#Kleiner als personal_best?
	kla=(cur_swarmval<personal_best_val)	#Maskieren
	personal_best_val[kla]=cur_swarmval[kla]
	personal_best_pos[kla]=swarmpos[kla]

	#Position von neigh_best
	if opt[2] > 0:
		for sw in range(n_swarm):
			ind = concatenate((personal_best_val,personal_best_val))[sw+1:sw+neigh_s+1].argmin()
			neigh_best_pos[sw] = personal_best_pos[ind]


	#Naechste Geschwindigkeit
	swarmvel = swarmvel*wk +  opt[0] * rand(n_swarm).reshape(n_swarm,1) * (personal_best_pos - swarmpos) + \
			opt[1] * rand(n_swarm).reshape(n_swarm,1) * ( global_best_pos - swarmpos) + \
			opt[2] * rand(n_swarm).reshape(n_swarm,1) * ( neigh_best_pos - swarmpos)

	#Geschwindigkeit begrenzen
	swarmvel[swarmvel> v_max]= v_max
	swarmvel[swarmvel<-v_max]=-v_max

	#Nacheste Position
	swarmpos += swarmvel

	#Positionen begrenzen
	swarmpos[swarmpos<sizef[:,0]]=array([sizef[:,0]]*n_swarm)[swarmpos<sizef[:,0]]
	swarmpos[swarmpos>sizef[:,1]]=array([sizef[:,1]]*n_swarm)[swarmpos>sizef[:,1]]

			
print "Global Best:" ,global_best_val, "@", global_best_pos

if plot==1: mlab.show()
